/*
Teem - an open source evolutionary software framework
http://lis.epfl.ch/resources/Teem
Stephane Magnenat <stephane.magnenat@epfl.ch>,
Antoine Beyeler <antoine.beyeler@epfl.ch>,
and other contributors. See AUTHORS for details
Laboratory of Intelligent Systems, EPFL, Lausanne

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ParameterGenotypeDecoder.h"
#include "../ga/ValueGenome.h"

/*!	\file ParameterGenotypeDecoder.cpp
	\brief Implementation of the genotype decoder that extract parameters from a genome.
*/


namespace Teem
{
	ParameterGenotypeDecoder::ParameterGenotypeDecoder(const std::string &root)
		: GenotypeDecoder(root)
	{
		Ishtar::Variable<unsigned> count(root + ".parameters.count", 0);

		for(unsigned i = 0; i < count; i++)
		{
			std::ostringstream oss;
			oss << root << ".parameters." << i;
			Ishtar::Variable<std::string> name(oss.str() + ".name");
			assert(name != "");
			Ishtar::Variable<double, false, 2> range(oss.str() + ".range");
			Ishtar::Variable<bool> logScale(oss.str() + ".log", false);
			
			parameters[name].range = An::Point(range[0], range[1]);
			parameters[name].log = logScale;
		}
	}

	Genome* ParameterGenotypeDecoder::createGenome()
	{
		// Create genome
		Ishtar::Variable<unsigned> defaultPrecision("teem.ga.defaultPrecision", 8);
		Genome* genome = Genome::create(Genome::BIT_GENOME, parameters.size() * defaultPrecision);
		genome->randomize();
		return genome;
	}

	Controller* ParameterGenotypeDecoder::decode(Genome *genome)
	{
		ValueGenome *vg = dynamic_cast<ValueGenome*>(genome);
		if (vg)
		{
			for(std::map<std::string, ParamDesc>::iterator i = parameters.begin(); i != parameters.end(); i++)
			{
				double val = vg->getDouble(i->second.range.x, i->second.range.y);
				if (i->second.log)
					val = pow(10, val);
				i->second.value = val;
			}
			
			vg->resetReading();
		}
		
		// We return no controllers.
		return NULL;
	}
}

