#ifndef __KHEPERA_NAVIGATION
#define __KHEPERA_NAVIGATION

#ifdef HAVE_ENKI

#include "SingleRobotExperiment.h"
#include "../world/World.h"
#include "../world/Entity.h"
#include "../utils/JustDoIt.h"
#include "../ga/GenerationalEvolution.h"

#include <an/Formula.h>
#include <an/Types.h>

#include <valarray>
#include <string>

class Viewer;

namespace Enki
{
	class Khepera;
}

namespace Teem
{
	class OctaveLogger;
	class Controller;
	
	//! Demonstration and education experiment recreating the battery recharging setup.
	class KheperaNavigation : public SingleRobotExperiment, public JustDoIt
	{
		friend class KheperaNavigationDrawer;
		
	protected:
	
		// preprocessing type
		enum Preproc
		{
			RAW = 0,
			SPATIAL_DIFF = 1,
			DIFF_AVERAGE = 2
		};
	
		SquareArenaWorld *w;		//!< world used by the experiment
		Khepera* robot;				//!< pointer to the robot
		Enki::Khepera* enkiInstance;	//!< instance of Enki::Khepera
		
		Ishtar::Variable<bool, true, 8> inputIR;			//!< use IR as input? (starting left then clockwise)
		Ishtar::Variable<bool, true, 2> inputEncoders;		//!< use encoder as input? (left, right)
		Ishtar::Variable<bool, true> inputVision;			//!< input vision
		
		Ishtar::Variable<unsigned, true> visionSize;		//!< size of the vision sensor
		Ishtar::Variable<unsigned, true> visionPreproc;		//!< preprocessing
		Ishtar::Variable<double, true> fieldOfView;			//!< robot field of view
		Ishtar::Variable<double> maxSpeed;					//!< max speed of the robot
		Ishtar::Variable<std::string, true> fitnessFunction;	//!< fitness function to use
		
		// hande coded mode.
		Ishtar::Variable<bool> handCodedMode;				//!< enable hand coded mode
		Ishtar::Variable<double, true, 9> leftWeights;		//!< synaptic weights for the left wheel
		Ishtar::Variable<double, true, 9> rightWeights;		//!< synaptic weights for the right wheel
		
		/* other desirable settings
		
			- evolution of morphology
			
		*/
	
		class World;
		
	public:
		//! Constructor
		KheperaNavigation();
		//! Destructor.
		virtual ~KheperaNavigation();
		
		virtual void setupDecoders(GenotypeDecoders &decoders);
		virtual void doIt();
	
	protected:
	
		Controller* controller;
		OctaveLogger* logger;
		An::Formula fitnessFormula;
		bool joystick;
		double leftJoystick, rightJoystick;
				
		// log and formula variables updated at the beginning of the time-step
		std::valarray<double> ir;
		std::valarray<double> vision;
		double leftEncoder, rightEncoder, leftMotor, rightMotor;
		double orientation;
		An::Point position;
		double instFitness;
		
		// SingleRobotExperiment overrides	
		virtual void createControllers(GenotypeDecoders &decoders, GenerationalPopulation &individuals, unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode);
		virtual void initEpoch(unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode);
		virtual void termEpoch(unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode);
		virtual double doSensoryMotorCycle(unsigned cycle, bool &kill);
		
		//! Assign inputs to the controller. The function uses the class variables that should contain up-to-date values.
		void assignInputsToController();
		
		//! Compute the current fitness. The function uses the class variables that should contain up-to-date values. 
		double computeFitness();
		
		//! Read vision sensor and preprocess.
		void readVisionSensor();
		
		//! Simulate the NN when using hand-coded weights.
		void runHandCodedNetwork(double &left, double &right);
	};
}

#endif //--HAVE_ENKI

#endif
