#ifndef __SINGLE_ROBOT_EXPERIMENT
#define __SINGLE_ROBOT_EXPERIMENT

#include "../ga/GenerationalEvolution.h"
#include "Experiment.h"


namespace Teem
{
	class SingleRobotExperiment : public Experiment, public Evaluable
	{
	protected:
	
		Ishtar::Variable<unsigned, true> epochCount;
		Ishtar::Variable<double, true> epochDuration;
		Ishtar::Variable<double, true> simulationStep;
		const unsigned cycleCount;
		
	public:
	
		//! Constructor
		SingleRobotExperiment();
		//! Destructor
		virtual ~SingleRobotExperiment() { }
		
		
		//virtual void setupDecoders(GenotypeDecoders &decoders);
		virtual void compute(GenotypeDecoders &genotypeDecoders, GenerationalPopulation &individuals, unsigned gen, unsigned ind, EvaluationMode mode);
	
	protected:
	
		//! Run a single epoch. Calls createControllers, initEpoch, doSensoryMotorCycle repeatedly and termEpoch.
		virtual double doEpoch(unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode);
		
		//! Create the controllers. Called at the beginning of the epoch.
		virtual void createControllers(GenotypeDecoders &decoders, GenerationalPopulation &individuals, unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode) { };
		
		//! Initialize the epoch. The controller are already created.
		virtual void initEpoch(unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode) { }
		
		//! Terminate the epoch.
		virtual void termEpoch(unsigned gen, unsigned ind, unsigned epoch, EvaluationMode mode) { }
		
		//! Execute a single sensory motor cycle. Called repeatedly by doEpoch.
		virtual double doSensoryMotorCycle(unsigned cycle, bool &kill) { return 0.0; }
	};
}

#endif
