/*
  Copyright (C) 2001-2005 Stephane Magnenat & Luc-Olivier de Charrière
  for any question or comment contact us at nct@ysagoon.com or nuage@ysagoon.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __SDLFONT_H
#define __SDLFONT_H

#include "GAGSys.h"
#include "GraphicContext.h"
#include "SDL_ttf.h"
#include <stack>

/*!	\file SDLFont.h
	\brief Interface of fonts using SDL
*/

namespace GAG
{
	//! TrueType font using SDL
	//! \ingroup libgag
	class SDLFont : public Font
	{
	private:
		friend class SDLGraphicContext;
		//! Private constructor
		SDLFont();
	
	public:
		//! Destructor, release TrueType font
		virtual ~SDLFont();
		//! Load filename TrueType font at size
		bool load(const char *filename, unsigned size);
		
		//! Return the width of string using this font
		int getStringWidth(const char *string) const;
		//! Return the height of string using this font
		int getStringHeight(const char *string) const;
		//! Return true if c is a printable character
		bool printable(char c) const;
		
		virtual void setColor(Uint8 r, Uint8 g, Uint8 b, Uint8 a = DrawableSurface::ALPHA_OPAQUE);
		virtual void pushColor(Uint8 r, Uint8 g, Uint8 b, Uint8 a = DrawableSurface::ALPHA_OPAQUE);
		virtual void popColor(void);
		virtual void getColor(Uint8 *r, Uint8 *g, Uint8 *b, Uint8 *a) const;
		
		virtual void setStyle(unsigned style);
		virtual void pushStyle(unsigned style);
		virtual void popStyle(void);
		virtual unsigned getStyle(void) const;
		
	protected:
		friend class SDLDrawableSurface;
		
		//! Draw text on Surface at (x,y) up to width w take accompt of clipping clip
		void drawString(SDL_Surface *Surface, int x, int y, int w, const char *text, SDL_Rect *clip=NULL) const;
		
	protected:
		TTF_Font *font; //!< TrueType font pointer
		
		std::stack<unsigned> styleStack; //!< stack of styles
		std::stack<SDL_Color> colorStack; //!< stack of colors
	};
}

#endif
