#ifndef __VERBOSELEVEL_H
#define __VERBOSELEVEL_H

#ifndef COMPILE_TIME_MAX_VERBOSE_LEVEL
//! Maximum verbose level if not already defined
#define COMPILE_TIME_MAX_VERBOSE_LEVEL 10
#endif

/*!	\file VerboseLevel.h
	\brief Interface to verbosity helpers
*/

namespace Teem
{
	/// Level of verbosity. 0 = no output. 1 = errors only.
	/// 2 = normal output. 3 = verbose. 4 = very verbose.
	extern unsigned verboseLevel;
	
	/// Locally change the verbosity. To be used as a stack-based
	/// instance for debug purpose.
	struct ChangeVerbosity
	{
		//! Constructor, set verboseLevel to v and store old one
		ChangeVerbosity(unsigned v) { oldVerbosity = verboseLevel; verboseLevel = v; }
		//! Destructor, restore old verbosity level
		~ChangeVerbosity() { verboseLevel = oldVerbosity; }
		
		unsigned oldVerbosity; //!< backup of old verbosity level
	};
}

//! Output s to standard output if lvl < Teem::verboseLevel
#define COUT(lvl, s) \
	do { if ((lvl <= COMPILE_TIME_MAX_VERBOSE_LEVEL) && (lvl <= ::Teem::verboseLevel)) { std::cout << s; } } while(false)
//! Output s to standard error if Teem::verboseLevel > 0
#define CERR(s) \
	do { if ((1 <= COMPILE_TIME_MAX_VERBOSE_LEVEL) && (1 <= ::Teem::verboseLevel)) { std::cerr << s; } } while(false)

#endif //__VERBOSELEVEL_H
