#include "Logger.h"

/*!	\file Logger.cpp
	\brief Implementation of logging helper classes
*/

namespace Teem
{
	Logger::~Logger()
	{
		for (LoggableMap::iterator i = loggables.begin(); i != loggables.end(); i++)
			for (size_t j = 0; j < i->second.size(); j++)
				delete i->second[j];
	}
	
	void Logger::add(const std::string &label, Loggable* loggable)
	{
		loggables[label].push_back(loggable);
	}
	
	void Logger::step()
	{
		if(!inited)
		{
			initLog(logFile);
			inited = true;
		}
		
		dumpSnapshot(logFile);
	}
	
	void OctaveLogger::initLog(std::ostream &log)
	{	
		if(!matlabMode)
			log << '%';
		
		for(LoggableMap::iterator i = loggables.begin(); i != loggables.end(); i++)
		{
			// label
			log << i->first;
			
			// column number indicator
			size_t num = i->second.size();
			if(num > 1)
				log << '#' << num;
			
			if(matlabMode)
				log << '\t';
			else
				log << ',';
		}
		log.seekp(-1, std::ios_base::cur);
		log << '\n';
	}
	
	void OctaveLogger::dumpSnapshot(std::ostream &log)
	{
		for(LoggableMap::iterator i = loggables.begin(); i != loggables.end(); i++)
		{
			// numbers
			for(size_t j = 0; j < i->second.size(); j++)
				log << *i->second[j] << (matlabMode ? '\t' : ',');
		}
		log.seekp(-1, std::ios_base::cur);
		log << '\n';
	}
}
