#include "SpikeDelayLine.h"

/*!	\file SpikeDelayLine.cpp
	\brief Implementation of a spike delay line
*/

namespace Teem
{
	SpikeDelayLine::SpikeDelayLine(double theDelay)
	{
		delay = theDelay;
		totalInterval = 0.0;
		timeSinceLastSpike = 0;
		spikePresent = false;
	}
	
	unsigned SpikeDelayLine::dumpOldSpikes()
	{
		if(!spikePresent)
			return 0;
			
		
		unsigned outSpikes = 0;
		
		/* Optimisation: disabled now
		// all spikes are old and have to be discarded
		if (timeSinceLastSpike >= delay)
		{
			outSpikes = intervals.size() + 1;
			intervals.erase();
			totalInterval = 0.0;
			spikePresent = false;
			return outSpikes;
		}
		*/
		
		// remove all the spike that are beyond the delay.
		while (totalInterval + timeSinceLastSpike >= delay)
		{
			if (totalInterval == 0.0)
			{
				spikePresent = false;
				timeSinceLastSpike = 0.0;
			}
			else
			{
				totalInterval -= intervals.front();
				intervals.pop();
			}
			
			++outSpikes;
		}
		
		return outSpikes;
	}
	
	
	unsigned SpikeDelayLine::step(double dt, bool pushSpike)
	{
		// increment time since last spike was pushed in the line
		if(spikePresent)
			timeSinceLastSpike += dt;
		
		// push the new spike in the queue
		if(pushSpike)
		{
			if(spikePresent)
			{
				intervals.push(timeSinceLastSpike);
				totalInterval += timeSinceLastSpike;
			}
			
			spikePresent = true;
			timeSinceLastSpike = 0.0;
		}
		
		// dump old spikes.
		return dumpOldSpikes();
	}
	
	unsigned SpikeDelayLine::setDelay(double theDelay, bool dumpSpikes)
	{
		delay = theDelay;
		if(dumpSpikes)
			return dumpOldSpikes();
		else
			return 0;
	}
}
