/*
Teem - an open source evolutionary software framework
http://lis.epfl.ch/resources/Teem
Stephane Magnenat <stephane.magnenat@epfl.ch>,
Antoine Beyeler <antoine.beyeler@epfl.ch>,
and other contributors. See AUTHORS for details
Laboratory of Intelligent Systems, EPFL, Lausanne

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "Genome.h"
#include "ValueGenome.h"
#include "SimpleGrowableGenome.h"
#include <assert.h>

/*!	\file Genome.cpp
	\brief Implementation of static methods for creating genomes
*/

namespace Teem
{
	Genome *Genome::create(GenomeType type, size_t size, bool disableAssertion)
	{
		Genome *ret;
		switch (type)
		{
			case BIT_GENOME:
			ret = new BitGenome(size);
			break;
			
			case SIMPLE_GROWABLE_GENOME:
			ret = new SimpleGrowableGenome(unsigned(size));
			break;
			
			default:
			if (!disableAssertion)
				assert(false);
			return NULL;
			break;
		}
		ret->disableAssertion = disableAssertion;
		return ret;
	}
	
	Genome *Genome::load(Ishtar::InputStream *stream, bool disableAssertion)
	{
		GenomeType type = static_cast<GenomeType>(stream->readUInt32());
		
		Genome *ret;
		switch (type)
		{
			case BIT_GENOME:
			ret = new BitGenome(stream);
			break;
			
			case SIMPLE_GROWABLE_GENOME:
			ret = new SimpleGrowableGenome(stream);
			break;
			
			default:
			if (!disableAssertion)
				assert(false);
			return NULL;
			break;
		}
		ret->disableAssertion = disableAssertion;
		return ret;
	}
}
