/*
Teem - an open source evolutionary software framework
http://lis.epfl.ch/resources/Teem
Stephane Magnenat <stephane.magnenat@epfl.ch>,
Antoine Beyeler <antoine.beyeler@epfl.ch>,
and other contributors. See AUTHORS for details
Laboratory of Intelligent Systems, EPFL, Lausanne

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __GENOME_H
#define __GENOME_H

#include <ishtar/settings.h>

/*!	\file Genome.h
	\brief Interfaces of abstract genomes
*/

namespace Teem
{
	//! The genome is basic class for genetic material containers
	//! \ingroup geneticalgos
	class Genome
	{
	public:
		//! Types of genomes available
		enum GenomeType
		{
			BIT_GENOME = 0, //!< genome made of bits
			SIMPLE_GROWABLE_GENOME, //!< genome made of nodes and links with floating point values
		};
		//! should we allays assert or do we handle the errors
		bool disableAssertion;
		//! did an error occur while loading the genome
		bool error;
		
	public:
		//! Virtual destructor, doing nothing
		virtual ~Genome() { }
		//! Randomize genome content
		virtual void randomize(void) = 0;
		//! Mutate genome content
		virtual void mutate(void) = 0;
		//! Cross genome content over other
		virtual void cross(const Genome *other) = 0;
		//! Clone this genome, returns the new genome
		virtual Genome *clone(void) = 0;
		
		//! Save the genome to a stream
		virtual void save(Ishtar::OutputStream *stream) = 0;
		
		//! Create a genome
		static Genome *create(GenomeType type, size_t size, bool disableAssertion = false);
		//! Load a genome from a stream
		static Genome *load(Ishtar::InputStream *stream, bool disableAssertion = false);
	};
}

#endif
